@@ -1089,12 +1089,18 @@ QQmlRefPointer<QQmlTypeData> QQmlTypeLoader::getType(const QByteArray &data, con
     return typeData;
 }
 
-QQmlRefPointer<QV4::CompiledData::CompilationUnit> QQmlTypeLoader::injectScript(
+static bool isModuleUrl(const QUrl &url)
+{
+    return url.path().endsWith(QLatin1String(".mjs"));
+}
+
+QQmlRefPointer<QV4::CompiledData::CompilationUnit> QQmlTypeLoader::injectModule(
         const QUrl &relativeUrl, const QV4::CompiledData::Unit *unit)
 {
     ASSERT_ENGINETHREAD();
 
-    QQmlRefPointer<QQmlScriptBlob> blob = QQml::makeRefPointer<QQmlScriptBlob>(relativeUrl, this);
+    QQmlRefPointer<QQmlScriptBlob> blob = QQml::makeRefPointer<QQmlScriptBlob>(
+            relativeUrl, this, QQmlScriptBlob::IsESModule::Yes);
 
     LockHolder<QQmlTypeLoader> holder(this);
     QQmlPrivate::CachedQmlUnit cached { unit, nullptr, nullptr};
@@ -1128,7 +1134,9 @@ QQmlRefPointer<QQmlScriptBlob> QQmlTypeLoader::getScript(
         scriptBlob = m_scriptCache.value(relativeUrl);
 
     if (!scriptBlob) {
-        scriptBlob = QQml::makeRefPointer<QQmlScriptBlob>(url, this);
+        scriptBlob = QQml::makeRefPointer<QQmlScriptBlob>(url, this, isModuleUrl(url)
+                ? QQmlScriptBlob::IsESModule::Yes
+                : QQmlScriptBlob::IsESModule::No);
         m_scriptCache.insert(url, scriptBlob);
 
         QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
