@@ -25,7 +25,15 @@ QQuickImageTextureProvider::QQuickImageTextureProvider()
 void QQuickImageTextureProvider::updateTexture(QSGTexture *texture) {
     if (m_texture == texture)
         return;
+
+    if (m_texture)
+        disconnect(m_texture, &QSGTexture::destroyed, this, nullptr);
+
     m_texture = texture;
+
+    if (m_texture)
+        connect(m_texture, &QSGTexture::destroyed, this, [this]() { updateTexture(nullptr); });
+
     emit textureChanged();
 }
 
@@ -67,6 +75,8 @@ QQuickImagePrivate::QQuickImagePrivate()
     to be scaled to that size. This behavior can be changed by setting the
     \l fillMode property, allowing the image to be stretched and tiled instead.
 
+    It is possible to provide \l {High Resolution Versions of Images}{"@nx" high DPI syntax}.
+
     \section1 Example Usage
 
     The following example shows the simplest usage of the Image type.
