@@ -10,7 +10,7 @@
 QT_BEGIN_NAMESPACE
 
 Q_LOGGING_CATEGORY(lcSGCurveProcessor, "qt.quick.curveprocessor");
-Q_LOGGING_CATEGORY(lcSGCurveIntersectionSolver, "qt.quick.curveprocessor.intersections");
+Q_STATIC_LOGGING_CATEGORY(lcSGCurveIntersectionSolver, "qt.quick.curveprocessor.intersections");
 
 namespace {
 // Input coordinate space is pre-mapped so that (0, 0) maps to [0, 0] in uv space.
@@ -94,7 +94,7 @@ static void fixWinding(TrianglePoints &p)
 // Return true if the determinant is negative, i.e. if the winding order is opposite of the triangle p1,p2,p3.
 // This means that p is strictly on the other side of p1-p2 relative to p3 [where p1,p2,p3 is a triangle with
 // a positive determinant].
-bool checkEdge(QVector2D &p1, QVector2D &p2, QVector2D &p, float epsilon)
+bool checkEdge(const QVector2D &p1, const QVector2D &p2, const QVector2D &p, float epsilon)
 {
     return determinant(p1, p2, p) <= epsilon;
 }
@@ -163,8 +163,8 @@ bool checkLineTriangleOverlap(TrianglePoints &triangle, LinePoints &line, float
 {
     // See if all vertices of the triangle are on the same side of the line
     bool s1 = determinant(line[0], line[1], triangle[0]) < 0;
-    auto s2 = determinant(line[0], line[1], triangle[1]) < 0;
-    auto s3 = determinant(line[0], line[1], triangle[2]) < 0;
+    bool s2 = determinant(line[0], line[1], triangle[1]) < 0;
+    bool s3 = determinant(line[0], line[1], triangle[2]) < 0;
     // If all determinants have the same sign, then there is no overlap
     if (s1 == s2 && s2 == s3) {
         return false;
@@ -224,8 +224,8 @@ static bool isIntersecting(const TrianglePoints &t1, const TrianglePoints &t2, Q
     constexpr int maxIterations = 7; // Maximum iterations allowed for Newton
 
     // Convert to double to get better accuracy.
-    QPointF td1[3] = { t1[0].toPointF(), t1[1].toPointF(), t1[2].toPointF() };
-    QPointF td2[3] = { t2[0].toPointF(), t2[1].toPointF(), t2[2].toPointF() };
+    std::array<QPointF, 3> td1 = { t1[0].toPointF(), t1[1].toPointF(), t1[2].toPointF() };
+    std::array<QPointF, 3> td2 = { t2[0].toPointF(), t2[1].toPointF(), t2[2].toPointF() };
 
     // F = P1(t1) - P2(t2) where P1 and P2 are bezier curve functions.
     // F = (0, 0) at the intersection.
@@ -1379,7 +1379,7 @@ bool QSGCurveProcessor::solveIntersections(QQuadPath &path, bool removeNestedPat
                 // For winding fill we take the left most path forward, so the inside stays on the right side
                 // For odd even fill we take the right most path forward so we cut of the smallest area.
                 // We come back at the intersection and add the missing pieces as subpaths later on.
-                if (t2 != 0 && t2 != 1) {
+                if (t1 !=0 && t1 != 1 && t2 != 0 && t2 != 1) {
                     QVector2D tangent1 = elem1.tangentAtFraction(t1);
                     if (!forward)
                         tangent1 = -tangent1;
@@ -1546,13 +1546,14 @@ void QSGCurveProcessor::processStroke(const QQuadPath &strokePath,
     auto triangles = customTriangulator2(thePath, penWidth, joinStyle, capStyle, miterLimit);
 
     auto addCurveTriangle = [&](const QQuadPath::Element &element, const TriangleData &t) {
+        QSGCurveStrokeNode::TriangleFlags flags;
+        flags.setFlag(QSGCurveStrokeNode::TriangleFlag::Line, element.isLine());
         addTriangle(t.points,
                     { element.startPoint(), element.controlPoint(), element.endPoint() },
-                    t.normals,
-                    element.isLine());
+                    t.normals, flags);
     };
 
-    auto addBevelTriangle = [&](const TrianglePoints &p)
+    auto addBevelTriangle = [&](const TrianglePoints &p, QSGCurveStrokeNode::TriangleFlags flags)
     {
         QVector2D fp1 = p[0];
         QVector2D fp2 = p[2];
@@ -1574,12 +1575,13 @@ void QSGCurveProcessor::processStroke(const QQuadPath &strokePath,
         n[0] = (p[0] - p[1]).normalized();
         n[2] = (p[2] - p[1]).normalized();
 
-        addTriangle(p, { fp1, QVector2D(0.0f, 0.0f), fp2 }, n, true);
+        flags.setFlag(QSGCurveStrokeNode::TriangleFlag::Line);
+        addTriangle(p, { fp1, QVector2D(0.0f, 0.0f), fp2 }, n, flags);
     };
 
-    for (const auto &triangle : triangles) {
+    for (const auto &triangle : std::as_const(triangles)) {
         if (triangle.pathElementIndex < 0) {
-            addBevelTriangle(triangle.points);
+            addBevelTriangle(triangle.points, {});
             continue;
         }
         const auto &element = thePath.elementAt(triangle.pathElementIndex);
