@@ -16,12 +16,14 @@
 #include <QtQuick/private/qquicktransition_p.h>
 #include <QtQuick/private/qquicktransitionmanager_p_p.h>
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 /*!
     \qmltype SwipeDelegate
     \inherits ItemDelegate
-//!     \instantiates QQuickSwipeDelegate
+//!     \nativetype QQuickSwipeDelegate
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols-delegates
@@ -68,7 +70,7 @@ QT_BEGIN_NAMESPACE
 
     \image qtquickcontrols-swipedelegate-behind.gif
 
-    \sa {Customizing SwipeDelegate}, {Delegate Controls}, {Qt Quick Controls 2 - Swipe to Remove}{Swipe to Remove Example}
+    \sa {Customizing SwipeDelegate}, {Delegate Controls}, {Qt Quick Controls 2 - Gallery}{Gallery Example}
 */
 
 namespace {
@@ -589,7 +591,7 @@ qreal QQuickSwipe::position() const
 void QQuickSwipe::setPosition(qreal position)
 {
     Q_D(QQuickSwipe);
-    const qreal adjustedPosition = qBound<qreal>(-1.0, position, 1.0);
+    const qreal adjustedPosition = std::clamp(position, qreal(-1.0), qreal(1.0));
     if (adjustedPosition == d->position)
         return;
 
@@ -771,17 +773,26 @@ bool QQuickSwipeDelegatePrivate::handleMouseMoveEvent(QQuickItem *item, QMouseEv
     if (!swipePrivate->left && !swipePrivate->right && !swipePrivate->behind)
         return false;
 
+    if (item != q && swipePrivate->complete) {
+        // If the delegate is swiped open, send the event to the exposed item,
+        // in case it's an interactive child (like a Button).
+        const auto posInItem = item->mapToItem(q, event->position().toPoint());
+        forwardMouseEvent(event, item, posInItem);
+    }
+
     // Don't handle move events for the control if it wasn't pressed.
     if (item == q && !pressed)
         return false;
 
-    const QPointF mappedEventPos = item->mapToItem(q, event->position().toPoint());
-    const qreal distance = (mappedEventPos - pressPoint).x();
+    const qreal distance = (event->globalPosition().x() != qInf() && event->globalPosition().y() != qInf()) ?
+                              (item->mapFromGlobal(event->globalPosition()) -
+                               item->mapFromGlobal(event->points().first().globalPressPosition())).x() : 0;
     if (!q->keepMouseGrab()) {
         // We used to use the custom threshold that QQuickDrawerPrivate::grabMouse used,
         // but since it's larger than what Flickable uses, it results in Flickable
         // stealing events from us (QTBUG-50045), so now we use the default.
-        const bool overThreshold = QQuickWindowPrivate::dragOverThreshold(distance, Qt::XAxis, event);
+        const bool overThreshold = QQuickDeliveryAgentPrivate::dragOverThreshold(distance,
+                                                                                 Qt::XAxis, event);
         if (window && overThreshold) {
             QQuickItem *grabber = q->window()->mouseGrabberItem();
             if (!grabber || !grabber->keepMouseGrab()) {
@@ -999,6 +1010,33 @@ QPalette QQuickSwipeDelegatePrivate::defaultPalette() const
     return QQuickTheme::palette(QQuickTheme::ListView);
 }
 
+/*! \internal
+    Recursively search right and/or left item tree of swipe delegate for any item that
+    contains the \a event position.
+
+    Returns the first such item found, otherwise \c nullptr.
+*/
+QQuickItem *QQuickSwipeDelegatePrivate::getPressedItem(QQuickItem *childItem, QMouseEvent *event) const
+{
+    if (!childItem || !event)
+        return nullptr;
+
+    QQuickItem *item = nullptr;
+
+    if (childItem->acceptedMouseButtons() &&
+            childItem->contains(childItem->mapFromScene(event->scenePosition()))) {
+        item = childItem;
+    } else {
+        const auto &childItems = childItem->childItems();
+        for (const auto &child: childItems) {
+            if ((item = getPressedItem(child, event)))
+                break;
+        }
+    }
+
+    return item;
+}
+
 QQuickSwipeDelegate::QQuickSwipeDelegate(QQuickItem *parent)
     : QQuickItemDelegate(*(new QQuickSwipeDelegatePrivate(this)), parent)
 {
@@ -1241,17 +1279,11 @@ void QQuickSwipeDelegate::mousePressEvent(QMouseEvent *event)
     swipePrivate->velocityCalculator.startMeasuring(event->position().toPoint(), event->timestamp());
 
     if (swipePrivate->complete) {
-        auto item = d->swipe.rightItem();
-        if (item && item->contains(item->mapFromScene(event->scenePosition()))) {
-            d->pressedItem = item;
-            d->handleMousePressEvent(item, event);
-        } else {
-            item = d->swipe.leftItem();
-            if (item && item->contains(item->mapFromScene(event->scenePosition()))) {
-                d->pressedItem = item;
-                d->handleMousePressEvent(item, event);
-            }
-        }
+        d->pressedItem = d->getPressedItem(d->swipe.rightItem(), event);
+        if (!d->pressedItem)
+            d->pressedItem = d->getPressedItem(d->swipe.leftItem(), event);
+        if (d->pressedItem)
+            d->handleMousePressEvent(d->pressedItem, event);
     }
 }
 
@@ -1382,7 +1414,7 @@ QQuickSwipeDelegateAttached::QQuickSwipeDelegateAttached(QObject *object)
         // the first one with an attached object.
         item->setAcceptedMouseButtons(Qt::AllButtons);
     } else {
-        qWarning() << "Attached properties of SwipeDelegate must be accessed through an Item";
+        qWarning() << "SwipeDelegate attached property must be attached to an object deriving from Item";
     }
 }
 
