@@ -3,10 +3,10 @@
 
 /*---
 esid: sec-array.prototype.indexof
-es5id: 15.4.4.14-8-3
+es5id: 15.4.4.14-8-2
 description: >
-    Array.prototype.indexOf returns -1 when abs('fromIndex') is length
-    of array - 1
+    Array.prototype.indexOf returns correct index when 'fromIndex' is
+    -1
 ---*/
 
-assert.sameValue([1, 2, 3, 4].indexOf(1, -3), -1, '[1, 2, 3, 4].indexOf(1, -3)');
+assert.sameValue([1, 2, 3, 4].indexOf(4, -1), 3, '[1, 2, 3, 4].indexOf(4, -1)');
