@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.2-2-8
+es5id: 15.2.3.2-2-13
 description: >
     Object.getPrototypeOf returns the [[Prototype]] of its parameter
-    (Math)
+    (RangeError)
 ---*/
 
-assert.sameValue(Object.getPrototypeOf(Math), Object.prototype, 'Object.getPrototypeOf(Math)');
+assert.sameValue(Object.getPrototypeOf(RangeError), Error, 'Object.getPrototypeOf(RangeError)');
