@@ -2,12 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.13-2-17
-description: >
-    Object.isExtensible returns true for all built-in objects
-    (Boolean.prototype)
+es5id: 15.2.3.13-2-7
+description: Object.isExtensible returns true for all built-in objects (Number)
 ---*/
 
-var e = Object.isExtensible(Boolean.prototype);
+var e = Object.isExtensible(Number);
 
 assert.sameValue(e, true, 'e');
