@@ -2,11 +2,11 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 12.1-7
-description: "12.1 - block '{ StatementListopt };' is not allowed: do-while"
+es5id: 12.1-5
+description: "12.1 - block '{ StatementListopt };' is not allowed: if-else-if"
 ---*/
 
 
 assert.throws(SyntaxError, function() {
-            eval("do{};while()");
+            eval("if{};else if{}");
 });
