@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "tst_qjsmanagedvalue.h"
 
@@ -146,13 +121,13 @@ void tst_QJSManagedValue::toString()
     {
         QJSManagedValue undefined(eng.toManagedValue(QVariant()));
         QCOMPARE(undefined.toString(), QStringLiteral("undefined"));
-        QCOMPARE(qjsvalue_cast<QString>(undefined), QString());
+        QCOMPARE(qjsvalue_cast<QString>(undefined), QStringLiteral("undefined"));
     }
 
     {
         QJSManagedValue null(eng.evaluate(QStringLiteral("null")), &eng);
         QCOMPARE(null.toString(), QStringLiteral("null"));
-        QCOMPARE(qjsvalue_cast<QString>(null), QString());
+        QCOMPARE(qjsvalue_cast<QString>(null), QStringLiteral("null"));
     }
 
     {
@@ -567,7 +542,7 @@ void tst_QJSManagedValue::toVariant()
     }
 
 
-    // array
+    // variant list
     {
         auto handler = qInstallMessageHandler([](QtMsgType type, const QMessageLogContext &, const QString &) {
             if (type == QtMsgType::QtWarningMsg)
@@ -578,12 +553,10 @@ void tst_QJSManagedValue::toVariant()
         QVariantList listIn;
         listIn << 123 << QStringLiteral("hello");
         QJSManagedValue array(eng.toManagedValue(listIn));
-        QVERIFY(array.isArray());
         QCOMPARE(array.property(QStringLiteral("length")).toInt(), 2);
 
         QVariant retained = array.toVariant();
-        QCOMPARE(retained.metaType(), QMetaType::fromType<QJSValue>());
-        QVERIFY(QJSManagedValue(retained.value<QJSValue>(), &eng).strictlyEquals(array));
+        QCOMPARE(retained.metaType(), QMetaType::fromType<QVariantList>());
 
         QVariantList listOut = retained.toList();
         QCOMPARE(listOut.size(), listIn.size());
@@ -591,7 +564,6 @@ void tst_QJSManagedValue::toVariant()
             QCOMPARE(listOut.at(i), listIn.at(i));
         // round-trip conversion
         QJSManagedValue array2(eng.toManagedValue(retained));
-        QVERIFY(array2.isArray());
         QCOMPARE(array2.property(QStringLiteral("length")).toInt(), array.property(QStringLiteral("length")).toInt());
         for (int i = 0; i < array.property(QStringLiteral("length")).toInt(); ++i)
             QVERIFY(array2.property(i).strictlyEquals(array.property(i)));
@@ -1550,8 +1522,6 @@ void tst_QJSManagedValue::strictlyEquals()
     {
         QJSManagedValue var1(eng.toManagedValue(QVariant(QStringList() << QStringLiteral("a"))));
         QJSManagedValue var2(eng.toManagedValue(QVariant(QStringList() << QStringLiteral("a"))));
-        QVERIFY(var1.isArray());
-        QVERIFY(var2.isArray());
         QVERIFY(!var1.strictlyEquals(var2));
     }
     {
@@ -1607,30 +1577,29 @@ void tst_QJSManagedValue::castToPointer()
 
 void tst_QJSManagedValue::engineDeleted()
 {
-    QJSEngine *eng = new QJSEngine;
+    std::unique_ptr<QJSEngine> eng = std::make_unique<QJSEngine>();
     QObject *temp = new QObject(); // Owned by JS engine, as newQObject() sets JS ownership explicitly
     QJSManagedValue v1(eng->toManagedValue(123));
     QCOMPARE(v1.type(), QJSManagedValue::Number);
     QJSManagedValue v2(eng->toManagedValue(QStringLiteral("ciao")));
     QCOMPARE(v2.type(), QJSManagedValue::String);
-    QJSManagedValue v3(eng->newObject(), eng);
+    QJSManagedValue v3(eng->newObject(), eng.get());
     QCOMPARE(v3.type(), QJSManagedValue::Object);
     QVERIFY(!v3.isNull());
-    QJSManagedValue v4(eng->newQObject(temp), eng);
+    QJSManagedValue v4(eng->newQObject(temp), eng.get());
     QCOMPARE(v4.type(), QJSManagedValue::Object);
     QVERIFY(!v4.isNull());
-    QJSManagedValue v5(QStringLiteral("Hello"), eng);
+    QJSManagedValue v5(QStringLiteral("Hello"), eng.get());
     QCOMPARE(v2.type(), QJSManagedValue::String);
 
-    delete eng;
+    eng.reset();
 
+    // You can still check the type, but anything involving the engine is obviously prohibited.
     QCOMPARE(v1.type(), QJSManagedValue::Undefined);
     QCOMPARE(v2.type(), QJSManagedValue::Undefined);
     QCOMPARE(v3.type(), QJSManagedValue::Undefined);
     QCOMPARE(v4.type(), QJSManagedValue::Undefined);
     QCOMPARE(v5.type(), QJSManagedValue::Undefined);
-
-    QVERIFY(v3.property(QStringLiteral("foo")).isUndefined());
 }
 
 void tst_QJSManagedValue::valueOfWithClosure()
@@ -1732,6 +1701,7 @@ void tst_QJSManagedValue::stringAndUrl()
     const QUrl url(string);
 
     const QJSManagedValue urlValue(engine.toManagedValue(url));
+    QVERIFY(urlValue.isUrl());
     QCOMPARE(urlValue.toString(), string);
     QCOMPARE(engine.fromManagedValue<QUrl>(urlValue), url);
 
@@ -1767,7 +1737,7 @@ void tst_QJSManagedValue::stringByIndex()
     const QString testString = QStringLiteral("foobar");
     QJSManagedValue str(testString, &engine);
 
-    for (uint i = 0; i < testString.length(); ++i) {
+    for (qsizetype i = 0; i < testString.size(); ++i) {
         QVERIFY(str.hasOwnProperty(i));
         QVERIFY(str.hasProperty(i));
 
@@ -1822,4 +1792,36 @@ void tst_QJSManagedValue::jsMetaTypes()
     QVERIFY(halfPopulated.property("ccc").isUndefined());
 }
 
+void tst_QJSManagedValue::exceptionsOnNullAccess()
+{
+    QJSEngine engine;
+    QJSManagedValue null(QJSValue(QJSValue::NullValue), &engine);
+    QJSManagedValue undef(QJSValue(QJSValue::UndefinedValue), &engine);
+
+    const QString nullReadError = engine.evaluate(
+                QStringLiteral("var n = null; n.prop")).toString();
+    const QString nullWriteError = engine.evaluate(
+                QStringLiteral("var n = null; n.prop = 5")).toString();
+    const QString undefReadError = engine.evaluate(
+                QStringLiteral("var n; n.prop")).toString();
+    const QString undefWriteError = engine.evaluate(
+                QStringLiteral("var n; n.prop = 5")).toString();
+
+    QVERIFY(null.property(QStringLiteral("prop")).isUndefined());
+    QVERIFY(engine.hasError());
+    QCOMPARE(engine.catchError().toString(), nullReadError);
+
+    null.setProperty(QStringLiteral("prop"), 5);
+    QVERIFY(engine.hasError());
+    QCOMPARE(engine.catchError().toString(), nullWriteError);
+
+    QVERIFY(undef.property(QStringLiteral("prop")).isUndefined());
+    QVERIFY(engine.hasError());
+    QCOMPARE(engine.catchError().toString(), undefReadError);
+
+    undef.setProperty(QStringLiteral("prop"), 5);
+    QVERIFY(engine.hasError());
+    QCOMPARE(engine.catchError().toString(), undefWriteError);
+}
+
 QTEST_MAIN(tst_QJSManagedValue)
